from django.test import TestCase

import jaseci.actions.live_actions as lact
import jaseci.tests.jac_test_code as jtc
from jaseci.actor.sentinel import sentinel
from jaseci.graph.graph import Graph
from jaseci.utils.utils import TestCaseHelper
from jaseci_serv.svc import MetaService


class JacTests(TestCaseHelper, TestCase):
    """Unit tests for Jac language"""

    def setUp(self):
        super().setUp()
        self.meta = MetaService()

    def tearDown(self):
        super().tearDown()

    def test_basic_use_calls_from_jac(self):
        """Test the execution of a basic walker building graph"""
        if not lact.load_remote_actions("http://js-use-qa"):
            self.skipTest("external resource not available")
        gph = Graph(m_id="anon", h=self.meta.hook())
        sent = sentinel(m_id=gph._m_id, h=gph._h)
        sent.register_code(jtc.prog1)
        test_node = sent.arch_ids.get_obj_by_name("life", kind="node").run()
        test_walker = sent.run_architype("use_test")
        test_walker.prime(test_node)
        report = test_walker.run()
        test_walker.save()
        test_walker._h.commit()
        self.assertEqual(len(report["report"][0]), 2)
        self.assertEqual(len(report["report"][0][1]), 2)

    def test_basic_use_single_string_calls_from_jac(self):
        """Test the execution of a basic walker building Graph"""
        if not lact.load_remote_actions("http://js-use-qa"):
            self.skipTest("external resource not available")
        gph = Graph(m_id="anon", h=self.meta.hook())
        sent = sentinel(m_id=gph._m_id, h=gph._h)
        sent.register_code(jtc.prog1)
        test_node = sent.arch_ids.get_obj_by_name("life", kind="node").run()
        test_walker = sent.run_architype("use_test_single")
        test_walker.prime(test_node)
        report = test_walker.run()
        test_walker.save()
        test_walker._h.commit()
        self.assertEqual(len(report["report"][0]), 1)
        self.assertEqual(len(report["report"][0][0]), 1)

    def test_use_qa_with_ctx(self):
        """Test the execution of a basic walker building Graph"""
        if not lact.load_remote_actions("http://js-use-qa"):
            self.skipTest("external resource not available")
        gph = Graph(m_id="anon", h=self.meta.hook())
        sent = sentinel(m_id=gph._m_id, h=gph._h)
        sent.register_code(jtc.prog1)
        test_node = sent.arch_ids.get_obj_by_name("life", kind="node").run()
        test_walker = sent.run_architype("use_test_with_ctx")
        test_walker.prime(test_node)
        report = test_walker.run()
        test_walker.save()
        test_walker._h.commit()
        self.assertEqual(len(report["report"][0]), 1)
        self.assertEqual(len(report["report"][0][0]), 1)

    def test_use_qa_with_ctx_clean(self):
        """Test the execution of a basic walker building Graph"""
        if not lact.load_remote_actions("http://js-use-qa"):
            self.skipTest("external resource not available")
        gph = Graph(m_id="anon", h=self.meta.hook())
        sent = sentinel(m_id=gph._m_id, h=gph._h)
        sent.register_code(jtc.prog1)
        test_node = sent.arch_ids.get_obj_by_name("life", kind="node").run()
        test_walker = sent.run_architype("use_test_with_ctx2")
        test_walker.prime(test_node)
        report = test_walker.run()
        test_walker.save()
        test_walker._h.commit()
        self.assertEqual(len(report["report"]), 8)
        self.assertEqual(len(report["report"][0]), 1)
        self.assertEqual(len(report["report"][0][0]), 1)



from jaseci.utils.utils import TestCaseHelper
from django.test import TestCase
from unittest.mock import patch
from django.core.management import call_command
from django.db.utils import OperationalError


class TestCmds(TestCaseHelper, TestCase):
    def setUp(self):
        super().setUp()

    def tearDown(self):
        super().tearDown()

    def test_wait_for_db_when_ready(self):
        """Test waiting for db when it's available"""
        with patch("django.db.utils.ConnectionHandler.__getitem__") as gi:
            gi.return_value = True
            call_command("wait_for_db")
            self.assertEqual(gi.call_count, 1)

    def test_wait_for_db_when_lagging(self):
        """Test waiting for db"""
        with patch("time.sleep") as ts:
            ts.return_value = True
            with patch("django.db.utils.ConnectionHandler.__getitem__") as gi:
                gi.side_effect = [OperationalError] * 5 + [True]
                call_command("wait_for_db")
                self.assertEqual(gi.call_count, 6)



from setuptools import setup, find_packages
from os.path import join


def get_ver():
    with open(join("./jaseci_serv", "VERSION")) as version_file:
        return version_file.read().strip()


setup(
    name="jaseci_serv",
    version=get_ver(),
    packages=find_packages(include=["jaseci_serv", "jaseci_serv.*"]),
    install_requires=[
        "jaseci",
        "Django>=3.2.12,<3.3.0",
        "djangorestframework>=3.13.1,<3.14.0",
        "django-rest-knox>=4.2.0,<4.3.0",
        "django-rest-passwordreset>=1.2.1,<1.3.0",
        "drf-yasg>=1.20.0,<1.21.0",
        "markdown>=3.3.6,<3.4.0",
        "psycopg2-binary>=2,<3",
        "sphinx>=2.4.3,<2.5.0",
        "django-cors-headers",
        "tblib",
        "django-celery-results>=2.3,<2.4",
        "django-celery-beat>=2.2",
    ],
    package_data={
        "": ["*.jac", "VERSION"],
    },
    entry_points={"console_scripts": ["jsserv = jaseci_serv.manage:main"]},
)



from django.contrib.auth import get_user_model
from django.urls import reverse

from rest_framework.test import APIClient
from rest_framework import status

from jaseci.utils.utils import TestCaseHelper
from django.test import TestCase


class PrivateJacAdminApiTests(TestCaseHelper, TestCase):
    """Test the authorized user node API"""

    def setUp(self):
        super().setUp()
        # First user is always super,
        self.user = get_user_model().objects.create_user(
            "throwawayJSCITfdfdEST_test@jaseci.com", "password"
        )
        self.nonadmin = get_user_model().objects.create_user(
            "JSCITfdfdEST_test@jaseci.com", "password"
        )
        self.client = APIClient()
        self.client.force_authenticate(self.user)
        self.notadminc = APIClient()
        self.notadminc.force_authenticate(self.nonadmin)
        self.master = self.user.get_master()

    def tearDown(self):
        super().tearDown()

    def test_jac_api_config_index_has_core(self):
        payload = {"op": "config_index"}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format="json"
        )
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertGreater(len(res.data), 2)
        self.assertIn("ACTION_SETS", res.data)

    def test_jac_api_create_config(self):
        """Test API for creating a config"""
        payload = {
            "op": "config_set",
            "name": "EMAIL_HOST_USER",
            "value": "val1",
            "do_check": False,
        }
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format="json"
        )
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        payload = {"op": "config_get", "name": "EMAIL_HOST_USER"}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format="json"
        )
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertEqual(res.data, "val1")

    def test_jac_api_create_config_needs_force(self):
        """Test API for creating a config"""
        payload = {"op": "config_delete", "name": "TEST"}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format="json"
        )
        payload = {"op": "config_set", "name": "TEST", "value": "val1"}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format="json"
        )
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        payload = {"op": "config_get", "name": "TEST"}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format="json"
        )
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertNotEqual(res.data, "val1")

    def test_jac_api_create_config_nonadmin_fails(self):
        """Test API for creating a config"""
        payload = {"op": "config_set", "name": "EMAIL_HOST_USER", "value": "val1"}
        res = self.notadminc.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format="json"
        )
        self.assertEqual(res.status_code, status.HTTP_403_FORBIDDEN)

    def test_jac_api_create_config_list(self):
        """Test API for creating a config"""
        payload = {
            "op": "config_set",
            "name": "EMAIL_HOST_USER",
            "value": "val1",
            "do_check": False,
        }
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format="json"
        )
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        payload = {
            "op": "config_set",
            "name": "EMAIL_HOST_PASSWORD",
            "value": "val2",
            "do_check": False,
        }
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format="json"
        )
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        payload = {
            "op": "config_set",
            "name": "EMAIL_DEFAULT_FROM",
            "value": "val3",
            "do_check": False,
        }
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format="json"
        )
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        payload = {
            "op": "config_set",
            "name": "EMAIL_BACKEND",
            "value": "val4",
            "do_check": False,
        }
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format="json"
        )
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        payload = {"op": "config_list"}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format="json"
        )
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertEqual(len(res.data), 4)
        self.assertIn("EMAIL_DEFAULT_FROM", res.data)



from django.urls import path, include

from . import views

app_name = "user_api"

urlpatterns = [
    path("create/", views.CreateUserView.as_view(), name="create"),
    path("create_super/", views.CreateSuperUserView.as_view(), name="create_super"),
    path("token/", views.CreateTokenView.as_view(), name="token"),
    path("manage/", views.ManageUserView.as_view(), name="manage"),
    path("manage/<int:id>", views.UpdateUserView.as_view(), name="update"),
    path(
        "logout_everyone/", views.LogoutAllUsersView.as_view(), name="logout_everyone"
    ),
    path("activate/<str:code>", views.ActivateUserView.as_view(), name="activate"),
    path(
        "password_reset/",
        include("django_rest_passwordreset.urls", namespace="password_reset"),
    ),
]
